@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* 
+/*
  * cal-backend-card-sexp.c
- * Copyright 1999, 2000, 2001, Ximian, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -24,7 +24,7 @@
 
 #include <string.h>
 #include <glib/gi18n-lib.h>
-#include <libedataserver/e-data-server-util.h>
+#include "libedataserver/e-data-server-util.h"
 #include <libecal/e-cal-time-util.h>
 
 #include "e-cal-backend-sexp.h"
@@ -45,6 +45,8 @@ struct _SearchContext {
 	gboolean occurs;
 };
 
+static ESExpResult *func_is_completed (ESExp *esexp, int argc, ESExpResult **argv, void *data);
+
 /**
  * e_cal_backend_sexp_func_time_now:
  * @esexp: An #ESExp object.
@@ -175,7 +177,7 @@ e_cal_backend_sexp_func_time_add_day (ESExp *esexp, int argc, ESExpResult **argv
 		return NULL;
 	}
 	n = argv[1]->value.number;
-	
+
 	result = e_sexp_result_new (esexp, ESEXP_RES_TIME);
 	result->value.time = time_add_day (t, n);
 
@@ -295,7 +297,7 @@ func_uid (ESExp *esexp, int argc, ESExpResult **argv, void *data)
 		return NULL;
 	}
 
-	arg_uid = argv[0]->value.string;	
+	arg_uid = argv[0]->value.string;
 	e_cal_component_get_uid (ctx->comp, &uid);
 
 	if (!arg_uid && !uid)
@@ -306,7 +308,7 @@ func_uid (ESExp *esexp, int argc, ESExpResult **argv, void *data)
 		equal = TRUE;
 	else
 		equal = FALSE;
-	
+
 	result = e_sexp_result_new (esexp, ESEXP_RES_BOOL);
 	result->value.bool = equal;
 
@@ -325,15 +327,15 @@ check_instance_time_range_cb (ECalComponent *comp, time_t instance_start, time_t
 }
 
 static icaltimezone *
-resolve_tzid_cb (const char *tzid, gpointer user_data)
+resolve_tzid (const char *tzid, gpointer user_data)
 {
 	SearchContext *ctx = user_data;
-                                                                                
+
         if (!tzid || !tzid[0])
                 return NULL;
         else if (!strcmp (tzid, "UTC"))
                 return icaltimezone_get_utc_timezone ();
-                                                                                
+
         return e_cal_backend_internal_get_timezone (ctx->backend, tzid);
 }
 
@@ -385,7 +387,7 @@ func_occur_in_time_range (ESExp *esexp, int argc, ESExpResult **argv, void *data
 	ctx->occurs = FALSE;
 	e_cal_recur_generate_instances (ctx->comp, start, end,
 					(ECalRecurInstanceFn) check_instance_time_range_cb,
-					ctx, resolve_tzid_cb, ctx,
+					ctx, resolve_tzid, ctx,
 					default_zone);
 
 	result = e_sexp_result_new (esexp, ESEXP_RES_BOOL);
@@ -394,6 +396,66 @@ func_occur_in_time_range (ESExp *esexp, int argc, ESExpResult **argv, void *data
 	return result;
 }
 
+static ESExpResult *
+func_due_in_time_range (ESExp *esexp, int argc, ESExpResult **argv, void *data)
+{
+	SearchContext *ctx = data;
+	time_t start, end;
+	ESExpResult *result;
+	icaltimezone *zone;
+	ECalComponentDateTime dt;
+	time_t due_t ;
+	gboolean retval;
+
+	/* Check argument types */
+
+	if (argc != 2) {
+		e_sexp_fatal_error (esexp, _("\"%s\" expects two arguments"),
+				"due-in-time-range");
+		return NULL;
+	}
+
+	if (argv[0]->type != ESEXP_RES_TIME) {
+		e_sexp_fatal_error (esexp, _("\"%s\" expects the first "
+					"argument to be a time_t"),
+				"due-in-time-range");
+		return NULL;
+	}
+
+	start = argv[0]->value.time;
+
+	if (argv[1]->type != ESEXP_RES_TIME) {
+		e_sexp_fatal_error (esexp, _("\"%s\" expects the second "
+					"argument to be a time_t"),
+				"due-in-time-range");
+		return NULL;
+	}
+
+	end = argv[1]->value.time;
+	e_cal_component_get_due (ctx->comp, &dt);
+
+	if(dt.value != NULL) {
+		zone = resolve_tzid (dt.tzid, ctx);
+		result = e_sexp_result_new (esexp, ESEXP_RES_INT);
+		if (zone)
+			due_t = icaltime_as_timet_with_zone(*dt.value,zone);
+		else
+			due_t = icaltime_as_timet(*dt.value);
+	}
+
+	if(dt.value != NULL && (due_t <= end && due_t >= start))
+		retval = TRUE;
+	else
+		retval = FALSE;
+
+	result = e_sexp_result_new (esexp, ESEXP_RES_BOOL);
+	result->value.bool = retval;
+
+	e_cal_component_free_datetime (&dt);
+
+	return result;
+}
+
 /* Returns whether a list of ECalComponentText items matches the specified string */
 static gboolean
 matches_text_list (GSList *text_list, const char *str)
@@ -446,6 +508,70 @@ matches_description (ECalComponent *comp, const char *str)
 	return matches;
 }
 
+static gboolean
+matches_attendee (ECalComponent *comp, const char *str)
+{
+	GSList *a_list = NULL, *l;
+	gboolean matches = FALSE;
+
+	e_cal_component_get_attendee_list (comp, &a_list);
+
+	for (l = a_list; l; l = l->next) {
+		ECalComponentAttendee *att = l->data;
+
+		if ((att->value && e_util_strstrcase (att->value, str)) || (att->cn != NULL &&
+					e_util_strstrcase (att->cn, str))) {
+			matches = TRUE;
+			break;
+		}
+	}
+
+	e_cal_component_free_attendee_list (a_list);
+
+	return matches;
+
+}
+
+static gboolean
+matches_organizer (ECalComponent *comp, const char *str)
+{
+
+	ECalComponentOrganizer org;
+
+	e_cal_component_get_organizer (comp, &org);
+	if (str && !*str)
+		return TRUE;
+
+	if ((org.value && e_util_strstrcase (org.value, str)) ||
+			(org.cn && e_util_strstrcase (org.cn, str)))
+		return TRUE;
+
+	return FALSE;
+}
+
+static gboolean
+matches_classification (ECalComponent *comp, const char *str)
+{
+	ECalComponentClassification classification;
+	ECalComponentClassification classification1;
+
+	if (!*str)
+		return FALSE;
+
+	if(g_str_equal (str, "Public"))
+		classification1 = E_CAL_COMPONENT_CLASS_PUBLIC;
+	else if(g_str_equal (str, "Private"))
+		classification1 = E_CAL_COMPONENT_CLASS_PRIVATE;
+	else if(g_str_equal (str, "Confidential"))
+		classification1 = E_CAL_COMPONENT_CLASS_CONFIDENTIAL;
+	else
+		classification1 = E_CAL_COMPONENT_CLASS_UNKNOWN;
+
+	e_cal_component_get_classification(comp, &classification);
+
+	return (classification == classification1 ? TRUE : FALSE);
+}
+
 /* Returns whether the summary in a component matches the specified string */
 static gboolean
 matches_summary (ECalComponent *comp, const char *str)
@@ -454,7 +580,7 @@ matches_summary (ECalComponent *comp, const char *str)
 
 	e_cal_component_get_summary (comp, &text);
 
-	if (str && !*str)
+	if (!*str)
 		return TRUE;
 
 	if (!text.value)
@@ -494,6 +620,92 @@ matches_any (ECalComponent *comp, const char *str)
 		|| matches_location (comp, str));
 }
 
+static gboolean
+matches_priority (ECalComponent *comp ,const char *pr)
+{
+	int *priority = NULL;
+
+	e_cal_component_get_priority (comp, &priority);
+
+	if (!priority || !*priority)
+		return FALSE;
+
+	if (g_str_equal (pr, "HIGH") && *priority <= 4)
+		return TRUE;
+	else if (g_str_equal (pr, "NORMAL") && *priority == 5)
+		return TRUE;
+	else if (g_str_equal (pr, "LOW") && *priority > 5)
+		return TRUE;
+	else if (g_str_equal (pr, "UNDEFINED") && (!priority || !*priority))
+		return TRUE ;
+
+	return FALSE;
+}
+
+static gboolean
+matches_status (ECalComponent *comp ,const char *str)
+{
+	icalproperty_status status ;
+
+	if (!*str)
+		return FALSE;
+
+	e_cal_component_get_status (comp, &status);
+
+	if (g_str_equal (str, "NOT STARTED") && status == ICAL_STATUS_NONE)
+			return TRUE;
+	else if (g_str_equal (str, "COMPLETED") && status == ICAL_STATUS_COMPLETED)
+			return TRUE;
+	else if(g_str_equal (str, "CANCELLED") && status == ICAL_STATUS_CANCELLED)
+			return TRUE;
+	else if(g_str_equal (str, "IN PROGRESS")  && status == ICAL_STATUS_INPROCESS)
+			return TRUE;
+
+	return FALSE;
+}
+
+static ESExpResult *
+func_has_attachment (ESExp *esexp, int argc, ESExpResult **argv, void *data)
+{
+	SearchContext *ctx = data;
+	ESExpResult *result;
+
+	if (argc != 0) {
+		e_sexp_fatal_error (esexp, _("\"%s\" expects no arguments"),
+				"has-attachments?");
+		return NULL;
+	}
+
+	result = e_sexp_result_new (esexp, ESEXP_RES_BOOL);
+	result->value.bool = e_cal_component_has_attachments (ctx->comp);
+
+	return result;
+}
+
+static ESExpResult *
+func_percent_complete (ESExp *esexp, int argc, ESExpResult **argv, void *data)
+{
+	SearchContext *ctx = data;
+	ESExpResult *result = NULL;
+	int *percent;
+
+	if (argc != 0) {
+		e_sexp_fatal_error (esexp, _("\"%s\" expects no arguments"),
+				"percent-completed");
+		return NULL;
+	}
+
+	e_cal_component_get_percent (ctx->comp, &percent);
+
+	if (percent && *percent) {
+		result = e_sexp_result_new (esexp, ESEXP_RES_INT);
+		result->value.number = *percent;
+
+	}
+
+	return result;
+}
+
 /* (contains? FIELD STR)
  *
  * FIELD - string, name of field to match (any, comment, description, summary, location)
@@ -547,10 +759,20 @@ func_contains (ESExp *esexp, int argc, ESExpResult **argv, void *data)
 		matches = matches_summary (ctx->comp, str);
 	else if (strcmp (field, "location") == 0)
 		matches = matches_location (ctx->comp, str);
+	else if (strcmp (field, "attendee") == 0)
+		matches = matches_attendee (ctx->comp, str);
+	else if (strcmp (field, "organizer") == 0)
+		matches = matches_organizer (ctx->comp, str);
+	else if(strcmp (field, "classification") == 0)
+		matches = matches_classification (ctx->comp, str);
+	else if(strcmp (field, "status") == 0)
+		matches = matches_status (ctx->comp, str);
+	else if(strcmp (field, "priority") == 0)
+		matches = matches_priority (ctx->comp, str);
 	else {
 		e_sexp_fatal_error (esexp, _("\"%s\" expects the first "
 					     "argument to be either \"any\", "
-					     "\"summary\", or \"description\", or \"location\""),
+					"\"summary\", or \"description\", or \"location\", or \"attendee\", or \"organizer\", or \"classification\""),
 				    "contains");
 		return NULL;
 	}
@@ -561,6 +783,35 @@ func_contains (ESExp *esexp, int argc, ESExpResult **argv, void *data)
 	return result;
 }
 
+/* (has-start?)
+ *
+ * A boolean value for components that have/don't have filled start date/time.
+ *
+ * Returns: a boolean indicating whether the component has start date/time filled or not.
+ */
+static ESExpResult *
+func_has_start (ESExp *esexp, int argc, ESExpResult **argv, void *data)
+{
+	SearchContext *ctx = data;
+	ESExpResult *result;
+	ECalComponentDateTime dt;
+
+	/* Check argument types */
+
+	if (argc != 0) {
+		e_sexp_fatal_error (esexp, _("\"%s\" expects no arguments"),
+				    "has-start");
+		return NULL;
+	}
+
+	e_cal_component_get_dtstart (ctx->comp, &dt);
+	result = e_sexp_result_new (esexp, ESEXP_RES_BOOL);
+	result->value.bool = dt.value != NULL;
+	e_cal_component_free_datetime (&dt);
+
+	return result;
+}
+
 /* (has-alarms?)
  *
  * A boolean value for components that have/dont have alarms.
@@ -635,7 +886,7 @@ func_has_alarms_in_range (ESExp *esexp, int argc, ESExpResult **argv, void *data
 		default_zone = icaltimezone_get_utc_timezone ();
 
 	alarms = e_cal_util_generate_alarms_for_comp (ctx->comp, start, end,
-						      omit, resolve_tzid_cb,
+						      omit, resolve_tzid,
 						      ctx, default_zone);
 
 	result = e_sexp_result_new (esexp, ESEXP_RES_BOOL);
@@ -775,7 +1026,7 @@ func_has_recurrences (ESExp *esexp, int argc, ESExpResult **argv, void *data)
 	}
 
 	result = e_sexp_result_new (esexp, ESEXP_RES_BOOL);
-	result->value.bool = e_cal_component_has_recurrences (ctx->comp);
+	result->value.bool = (e_cal_component_has_recurrences (ctx->comp) || e_cal_component_is_instance (ctx->comp));
 
 	return result;
 }
@@ -941,12 +1192,12 @@ entry_compare(SearchContext *ctx, struct _ESExp *f,
 			if (any_field
 			    || !strcmp (prop_info_table[i].query_prop, propname)) {
 				info = &prop_info_table[i];
-				
+
 				if (info->prop_type == PROP_TYPE_NORMAL) {
 					char *prop = NULL;
 					/* searches where the query's property
 					   maps directly to an ecard property */
-					
+
 					prop = e_card_simple_get (ctx->card, info->field_id);
 
 					if (prop && compare(prop, argv[1]->value.string)) {
@@ -963,7 +1214,7 @@ entry_compare(SearchContext *ctx, struct _ESExp *f,
 					const char *prop = NULL;
 					/* searches where the query's property
 					   maps directly to an ecard property */
-					
+
 					prop = e_card_get_id (ctx->card->card);
 
 					if (prop && compare(prop, argv[1]->value.string)) {
@@ -982,7 +1233,7 @@ entry_compare(SearchContext *ctx, struct _ESExp *f,
 					break;
 			}
 		}
-		
+
 	}
 	r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 	r->value.bool = truth;
@@ -1004,17 +1255,20 @@ static struct {
 	{ "time-add-day", e_cal_backend_sexp_func_time_add_day, 0 },
 	{ "time-day-begin", e_cal_backend_sexp_func_time_day_begin, 0 },
 	{ "time-day-end", e_cal_backend_sexp_func_time_day_end, 0 },
-
 	/* Component-related functions */
 	{ "uid?", func_uid, 0 },
 	{ "occur-in-time-range?", func_occur_in_time_range, 0 },
+	{ "due-in-time-range?", func_due_in_time_range, 0 },
 	{ "contains?", func_contains, 0 },
+	{ "has-start?", func_has_start, 0 },
 	{ "has-alarms?", func_has_alarms, 0 },
 	{ "has-alarms-in-range?", func_has_alarms_in_range, 0 },
 	{ "has-recurrences?", func_has_recurrences, 0 },
 	{ "has-categories?", func_has_categories, 0 },
 	{ "is-completed?", func_is_completed, 0 },
-	{ "completed-before?", func_completed_before, 0 }
+	{ "completed-before?", func_completed_before, 0 },
+	{ "has-attachments?", func_has_attachment, 0 },
+	{ "percent-complete?", func_percent_complete, 0 }
 };
 
 /**
@@ -1080,7 +1334,7 @@ e_cal_backend_sexp_match_object (ECalBackendSExp *sexp, const char *object, ECal
 
 	comp = e_cal_component_new ();
 	e_cal_component_set_icalcomponent (comp, icalcomp);
-	
+
 	retval = e_cal_backend_sexp_match_comp (sexp, comp, backend);
 
 	g_object_unref (comp);
@@ -1141,7 +1395,7 @@ const char *
 e_cal_backend_sexp_text (ECalBackendSExp *sexp)
 {
 	ECalBackendSExpPrivate *priv;
-	
+
 	g_return_val_if_fail (sexp != NULL, NULL);
 	g_return_val_if_fail (E_IS_CAL_BACKEND_SEXP (sexp), NULL);
 
