@@ -43,10 +43,13 @@ typedef struct _CamelVeeMessageInfo CamelVeeMessageInfo;
 struct _CamelVeeMessageInfo {
 	CamelMessageInfo info;
 	CamelFolderSummary *summary;
+	guint32 old_flags;  /* These are just for identifying changed flags */
 };
 
 struct _CamelVeeSummary {
 	CamelFolderSummary summary;
+	gboolean force_counts;
+	guint32 fake_visible_count;
 };
 
 struct _CamelVeeSummaryClass {
