@@ -1,15 +1,15 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* camel-groupwise-provider.c: Groupwise provider registration code */
+/* camel-groupwise-provider.c: GroupWise provider registration code */
 
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *           Sivaiah Nallagatla <snallagatla@novell.com>
  *           Rodrigo Moya <rodrigo@ximian.com>
  *
- *  Copyright 2003 Novell, Inc. (www.novell.com)
+ *  Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,10 +17,10 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
@@ -29,12 +29,16 @@
 #endif
 
 #include <string.h>
+
+#include <glib.h>
+#include <glib/gi18n-lib.h>
 #include <gmodule.h>
+
 #include "camel-provider.h"
+#include "camel-sasl.h"
 #include "camel-session.h"
 #include "camel-url.h"
-#include "camel-sasl.h"
-#include "camel-i18n.h"
+
 #include "camel-groupwise-store.h"
 #include "camel-groupwise-transport.h"
 
@@ -50,21 +54,21 @@ CamelProviderConfEntry groupwise_conf_entries[] = {
 	{ CAMEL_PROVIDER_CONF_SECTION_START, "mailcheck", NULL,
 	  N_("Checking for new mail") },
 	{ CAMEL_PROVIDER_CONF_CHECKBOX, "check_all", NULL,
-	  N_("Check for new messages in all folders"), "1" },
+	  N_("C_heck for new messages in all folders"), "1" },
 	{ CAMEL_PROVIDER_CONF_SECTION_END },
 
 	{ CAMEL_PROVIDER_CONF_SECTION_START, "general", NULL, N_("Options") },
 	{ CAMEL_PROVIDER_CONF_CHECKBOX, "filter", NULL,
-	  N_("Apply filters to new messages in Inbox on this server"), "0" },
+	  N_("_Apply filters to new messages in Inbox on this server"), "0" },
 	{ CAMEL_PROVIDER_CONF_CHECKBOX, "filter_junk", NULL,
-	  N_("Check new messages for Junk contents"), "0" },
+	  N_("Check new messages for J_unk contents"), "0" },
 	{ CAMEL_PROVIDER_CONF_CHECKBOX, "filter_junk_inbox", "filter_junk",
-	  N_("Only check for Junk messages in the INBOX folder"), "0" },
+	  N_("Only check for Junk messages in the IN_BOX folder"), "0" },
 	{ CAMEL_PROVIDER_CONF_CHECKBOX, "sync_offline", NULL,
-	  N_("Automatically synchronize account locally"), "0" },
+	  N_("Automatically synchroni_ze account locally"), "0" },
 	{ CAMEL_PROVIDER_CONF_SECTION_END },
 
-	/* extra Groupwise  configuration settings */
+	/* extra GroupWise  configuration settings */
 	{CAMEL_PROVIDER_CONF_SECTION_START, "soapport", NULL,
 	  N_("SOAP Settings") },
 
@@ -84,12 +88,12 @@ static CamelProvider groupwise_provider = {
 	"groupwise",
 	N_("Novell GroupWise"),
 
-	N_("For accessing Novell Groupwise servers"),
+	N_("For accessing Novell GroupWise servers"),
 
 	"mail",
 
 	CAMEL_PROVIDER_IS_REMOTE | CAMEL_PROVIDER_IS_SOURCE |
-	CAMEL_PROVIDER_IS_STORAGE | CAMEL_PROVIDER_SUPPORTS_SSL,
+	CAMEL_PROVIDER_IS_STORAGE | CAMEL_PROVIDER_SUPPORTS_SSL | CAMEL_PROVIDER_DISABLE_SENT_FOLDER,
 
 	CAMEL_URL_NEED_USER | CAMEL_URL_NEED_HOST | CAMEL_URL_ALLOW_AUTH,
 
@@ -134,11 +138,12 @@ camel_provider_module_init(void)
 	groupwise_provider.url_equal = groupwise_url_equal;
 	groupwise_provider.auto_detect = groupwise_auto_detect_cb;
 	groupwise_provider.authtypes = g_list_prepend (groupwise_provider.authtypes, &camel_groupwise_password_authtype);
+	groupwise_provider.translation_domain = GETTEXT_PACKAGE;
 	
 	if (use_imap)
 		groupwise_provider.object_types[CAMEL_PROVIDER_STORE] = imap_provider->object_types [CAMEL_PROVIDER_STORE];
 	else 	{
-		groupwise_provider.object_types[CAMEL_PROVIDER_STORE] =  camel_groupwise_store_get_type() ;
+		groupwise_provider.object_types[CAMEL_PROVIDER_STORE] =  camel_groupwise_store_get_type();
 		groupwise_provider.object_types[CAMEL_PROVIDER_TRANSPORT] = camel_groupwise_transport_get_type();
 	} 
 	
@@ -160,7 +165,6 @@ groupwise_url_hash (gconstpointer key)
 	guint hash = 0;
 
 	add_hash (&hash, u->user);
-	add_hash (&hash, u->authmech);
 	add_hash (&hash, u->host);
 	hash ^= u->port;
 	
@@ -188,8 +192,8 @@ groupwise_url_equal (gconstpointer a, gconstpointer b)
 {
 	const CamelURL *u1 = a, *u2 = b;
 	
-	return check_equal (u1->user, u2->user)
-		&& check_equal (u1->authmech, u2->authmech)
+	return check_equal (u1->protocol, u2->protocol)
+		&& check_equal (u1->user, u2->user)
 		&& check_equal (u1->host, u2->host)
 		&& u1->port == u2->port;
 }
