@@ -321,10 +321,13 @@ mbox_get_message(CamelFolder *folder, const gchar * uid, CamelException *ex)
 	CamelLocalFolder *lf = (CamelLocalFolder *)folder;
 	CamelMimeMessage *message = NULL;
 	CamelMboxMessageInfo *info;
+	CamelMessageInfoBase *mi = (CamelMessageInfoBase *)info;
+	char *xev;
 	CamelMimeParser *parser = NULL;
 	int fd, retval;
 	int retried = FALSE;
 	off_t frompos;
+	guint32 flags;
 
 	d(printf("Getting message %s\n", uid));
 
@@ -424,6 +427,6 @@ fail:
 		camel_object_trigger_event((CamelObject *)folder, "folder_changed", lf->changes);
 		camel_folder_change_info_clear(lf->changes);
 	}
-	
+
 	return message;
 }
