@@ -246,12 +246,13 @@ term_eval_and(struct _ESExp *f, int argc, struct _ESExpTerm **argv, void *data)
 	int type=-1;
 	int bool = TRUE;
 	int i;
+	char *oper;
 
 	r(printf("( and\n"));
 
 	r = e_sexp_result_new(f, ESEXP_RES_UNDEFINED);
 
-	char *oper = "AND";
+	oper = "AND";
 	f->operators = g_slist_prepend (f->operators, oper);
 
 	for (i=0;bool && i<argc;i++) {
@@ -308,10 +309,11 @@ term_eval_or(struct _ESExp *f, int argc, struct _ESExpTerm **argv, void *data)
 	int type = -1;
 	int bool = FALSE;
 	int i;
+	char *oper;
 
 	r(printf("(or \n"));
 
-	char *oper = "OR";
+	oper = "OR";
 	f->operators = g_slist_prepend (f->operators, oper);
 
 	r = e_sexp_result_new(f, ESEXP_RES_UNDEFINED);
@@ -704,7 +706,7 @@ e_sexp_term_eval(struct _ESExp *f, struct _ESExpTerm *t)
 		r->value.time = t->value.time;
 		break;
 	case ESEXP_TERM_IFUNC:
-		if (t->value.func.sym->f.ifunc)
+		if (t->value.func.sym && t->value.func.sym->f.ifunc)
 			r = t->value.func.sym->f.ifunc(f, t->value.func.termcount, t->value.func.terms, t->value.func.sym->data);
 		break;
 	case ESEXP_TERM_FUNC:
@@ -1031,6 +1033,8 @@ parse_list(ESExp *f, int gotbrace)
 		case G_TOKEN_IDENTIFIER:
 			e_sexp_fatal_error(f, "Unknown identifier: %s", g_scanner_cur_value(gs).v_identifier);
 			break;
+		case G_TOKEN_LEFT_PAREN:
+			return parse_list(f, TRUE);
 		default:
 			e_sexp_fatal_error(f, "Unexpected token encountered: %d", token);
 		}
