@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* e-source-group.h
  *
- * Copyright (C) 2003  Ximian, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU Lesser General Public
@@ -113,6 +113,9 @@ void         e_source_group_foreach_property (ESourceGroup *source,
 
 char *e_source_group_to_xml (ESourceGroup *group);
 
+gboolean e_source_group_equal (ESourceGroup *a, ESourceGroup *b); 
+gboolean e_source_group_xmlstr_equal (const gchar *a, const gchar *b); 
+
 G_END_DECLS
 
 #endif /* _E_SOURCE_GROUP_H_ */
