@@ -2,7 +2,7 @@
 
 /* e-book-auth-util.c - Lame helper to load addressbooks with authentication.
  *
- * Copyright (C) 2005 Novell, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU Lesser General Public
@@ -25,7 +25,7 @@
 
 #include <config.h>
 #include <string.h>
-#include <gtk/gtkmessagedialog.h>
+#include <gtk/gtk.h>
 #include <glib/gi18n-lib.h>
 #include <libebook/e-book.h>
 #include <libedataserverui/e-passwords.h>
@@ -72,9 +72,10 @@ load_source_auth_cb (EBook *book, EBookStatus status, gpointer closure)
 {
 	LoadSourceData *data = closure;
 
-	if (status != E_BOOK_ERROR_OK) {
+	switch (status) {
+
 		/* the user clicked cancel in the password dialog */
-		if (status == E_BOOK_ERROR_CANCELLED) {
+		case E_BOOK_ERROR_CANCELLED:
 			if (e_book_check_static_capability (book, "anon-access")) {
 				GtkWidget *dialog;
 
@@ -89,21 +90,16 @@ load_source_auth_cb (EBook *book, EBookStatus status, gpointer closure)
 								 _("Accessing LDAP Server anonymously"));
 				g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
 				gtk_widget_show (dialog);
-				if (data->open_func)
-					data->open_func (book, E_BOOK_ERROR_OK, data->open_func_data);
-				free_load_source_data (data);
-				return;
 			}
-		} else if (status == E_BOOK_ERROR_INVALID_SERVER_VERSION) {
-#if 0
-			e_error_run (NULL, "addressbook:server-version", NULL);
-#endif
+			break;
+
+		case E_BOOK_ERROR_INVALID_SERVER_VERSION:
 			status = E_BOOK_ERROR_OK;
-			if (data->open_func)
-				data->open_func (book, status, data->open_func_data);
-			free_load_source_data (data);
-			return;
-		} else {
+			break;
+
+		case E_BOOK_ERROR_AUTHENTICATION_FAILED:
+		case E_BOOK_ERROR_AUTHENTICATION_REQUIRED:
+		{
 			const gchar *uri = e_book_get_uri (book);
 			gchar *stripped_uri = remove_parameters_from_uri (uri);
 			const gchar *auth_domain = e_source_get_property (data->source, "auth-domain");
@@ -111,13 +107,18 @@ load_source_auth_cb (EBook *book, EBookStatus status, gpointer closure)
 
 			component_name = auth_domain ? auth_domain : "Addressbook";
 
-			e_passwords_forget_password (component_name, stripped_uri);
+			if (status == E_BOOK_ERROR_AUTHENTICATION_FAILED)
+				e_passwords_forget_password (component_name, stripped_uri);
+
 			addressbook_authenticate (book, TRUE, data->source, load_source_auth_cb, closure);
 
 			g_free (stripped_uri);
 
 			return;
 		}
+
+		default:
+			break;
 	}
 
 	if (data->open_func)
